function idprops(type)
%IDPROPS  Help on IDENT model and data properties.
%
%   IDPROPS gives details on the generic properties of IDMODEL models.
%
%   IDPROPS(TYPE) gives details on the properties specific
%   to the various types of models and data.  The string TYPE 
%   selects the model type among the following:
%
%      'iddata':     The basic data representation (IDDATA object)
%      'idmodel':    The basic model representation
%      'idpoly':     Polynomial models Ay=[B/F]u+[C/D]e (IDPOLY object)
%      'idarx':      Multivariable ARX models Ay=Bu+e (IDARX object)
%      'idss':       State-space models (IDSS object)
%      'idgrey':     User defined linear models (IDGREY object)
%      'idproc':     Process models (like gain + time constant + delay)
%      'idfrd':      Identified Frequency Response data (IDFRD object)
%      'algorithm':  Properties associated with the algorithm.
%      'estimation': Properties associated with the estimation result
%      'focus':      Focus estimation fit to certain frequency ranges
%   
%   Note that you can type
%      idprops idpoly
%   as shorthand for
%      idprops('idpoly'), and also use just the first letters.
%
%   See also IDHELP, SET, GET, IDPOLY, IDARX, IDSS, IDGREY, IDDATA, IDFRD

%   Copyright 1986-2003 The MathWorks, Inc.
%   $Revision: 1.21.4.1 $  $Date: 2004/04/10 23:19:02 $


if nargin==0,
   help idprops%type = 'idmodel';
   return
elseif ~ischar(type)
   error('TYPE must be a string.')
end
try
type = pnmatchd(type,{'idpoly','idss','idgrey','idproc','idfrd','iddata',...
        'idmodel','idarx','estimation','algorithm','focus'});
catch
    error(lasterr)
end
% if length(type)<3
%     error('You must supply at least three letters of the argument')
% end
% if length(type)==3&strcmp(lower(type),'idp')
%     error('Please make clear is you mean ''IDPOLY'' or ''IDPROC''.')
% end
switch type
case 'gen'
   %DisplayStr ={...
   %      ' General 
case 'idmodel'
   % Generic IDMODEL properties
   DisplayStr = {...
         ' Help on properties of IDMODEL models.'...
         ' '...
         ' IDMODEL properties are divided into two categories:' ...
         '   * Generic properties shared by all IDMODEL model types'...
         '   * Properties specific to IDPOLY, IDARX, IDSS, or IDGREY models.'...
         '     For details, type IDPROPS followed by IDPOLY, IDARX, IDSS, or IDGREY.'...
         '  '...
         ' For an IDMODEL model with Ny outputs and Nu inputs, the generic'... 
         ' IDMODEL properties include:'...
         ' '...
         ' Name          - An optional value to identify the model. A string.'...
         ' '...
         ' Ts            - Sample time (unit = ''TimeUnit'').'...
         '                 Assignable values:'...
         '                   * Positive scalar (sampling period) for discrete models.'...
         '                   * 0 for continuous models.'...
         '                 Default value:  1.'...
         '  '...
         ' ParameterVector - The vector of estimated or nominal parameters.'...
         '                   Assignable values:'...
         '                     * A column vector of adequate length Np.'...
         '                   NOTE: Typically this property is automatically set by'...
         '                   estimation algorithms.'...
       ' '...
         ' PName -           Contains the names of the parameters.'...
         '                   Content: A Np-by-1 cell array of strings. See SETPNAME for'...
	     '                   Automatic generation of mnemonic parameter names.'...
           '  '...
         ' CovarianceMatrix - The covariance matrix of the ParameterVector.'...
         '                    Assignable values:'...
         '                     * An Np-by-Np matrix'...
         '                    NOTE: Typically this property is automatically set by'...
         '                    estimation algorithms.'...
         '                    It can also be set to ''None'' to inhibit all',...
         '                    calculation of uncertainties. This may save time.'...
		 '                    If CovarianceMatrix has been set to ''None'', setting'...
		 '                    it to ''Estimate'' will again facilitate covariance',...
		 '                    estimation, as in '...
		 '                    M = N4SID(Dat,4,''cov'',''none''), M = PEM(Dat,M,''cov'',''est'')'...
         '  '...
         ' NoiseVariance - The variance (covariance matrix) if the model innovations e.'...
         '                 Assignable values:'...
         '                   * An Ny-by-Ny matrix'...
         '  '...
         ' Algorithm     - A structure defining the algorithm used for estimation.'...
         '                 Type ''idprops Algorithm'' for details.'...
         '  '...
         ' EstimationInfo - A structure with information from the estimation process.'...
         '                 Type ''idprops Estimation'' for details.'...
         '  '...
         ' InputDelay    - A column vector of the same length as the number of'...
         '                 inputs. Defines the shifts that should be applied to'...
         '                 the input sequence to obtain the desired response.'...
         '                 Note that unlike the property ''nk'' for IDPOLY and IDSS'...
         '                 InputDelay does not show up in the model matrices and'...
         '                 transfer functions. For discrete time models (Ts>0)'... 
         '                 InputDelay is given as number of samples, while for'...
         '                 continuous time models it is measured in TimeUnit.'...
         '                 Negative InputDelay gives non-causal models.'...
         '   '...
         ' InputName     - Input channel names.'...
         '                 Content:  Nu-by-1 cell array of strings.'...
         '                 Assignable values:'...
         '                   * String for single-input models, e.g., ''torque'''...
         '                   * Cell vector of strings for multiple-input models,'...
         '                     e.g., {''thrust'' ; ''aileron deflection''}'...
         '                   Default names:  {''u1'';''u2''; ...}.'...
         '  '...
         ' OutputName    - Output channel names.'...
         '                 Content:  Ny-by-1 cell array of strings.'...
         '                 Assignable/default values: Analogous to InputName.'...
         '  '...
         ' InputUnit     - Input channel units.'...
         '                 Content:  Nu-by-1 cell array of strings.'...
         '                 Assignable values:'...
         '                   * String for single-input models, e.g., ''m/s'''...
         '                   * Cell vector of strings for multiple-input models,'...
         '                     e.g., {''^o C'' ; ''Newton''}'...
         '                   Default names:  Empty strings.'...
         '  '...
         ' OutputUnit    - Output channel units.'...
         '                 Content:  Ny-by-1 cell array of strings.'...
         '                 Assignable/default values: Analogous to InputUnit.'...
         '  '...
         ' TimeUnit     - Unit for sampling interval and time scale.'...
         '  '...
         ' Notes         - User''s notes.'...
         '                 Content:  Any string or cell array of strings.'...
         '                 Example:  ''This model was created on Jan. 1st, 2000'''...
         '  '...
         ' Userdata      - Additional information or data.'...
         '                 Content:  Any MATLAB data type.'...
         ' '...
         ' Use SET and GET to set and access property values, as in'...
         '    set(model,''inputname'',''u'')'...
         '    get(model,''Algorithm'')'...
         ' Autofill, case insensitivity and subreferencing applies to Property names'...
         ' as in model.noi or model.pa.'...
         [' NOTE: For quick reference ''Input'' and ''Output'' can' ...
	  ' replaced by']...
       ' ''u'' and ''y'' as in model.una or model.yu.'...
      };
   
case 'algorithm'
   DisplayStr = {...
         ' Help on algorithmic properties for IDMODEL models.'...
         ' '...
         ' The property ''Algorithm'' contains the following fields which can be'...
         ' individually accessed by GET and SET and subscripting:'...
         ' '...
         ' Focus        - The focus of the fit when estimating the model.'...
         '                Assignable values:'...
         '                   * ''Prediction'' (default) Focus on producing good'... 
         '                       predictors'...
         '                   * ''Simulation'' Focus on making a good fit for'...
         '                       simulation (with the current input)'...
         '                       This guarantees a stable model.'...
		 '                   * ''Stability'' to enforce the resulting model to be'...
		 '                       stable, but still having a prediction focus.'...
         '                   * A matrix of the type [WL1 WH1;WL2 WH2;...]'...
         '                       where each row defines a pass band to which'...
         '                       the fit is concentrated. Note that these'...
         '                       frequencies are given in rad/s.'...
         '                   * Any SISO filter in LTI or IDMODEL format.'...
         '                       The model is fitted to the data with a frequency '...
         '                       weighting of this SISO filter''s response.'...
         '                       The filter can also be given as {A,B,C,D} for'...
         '                       state-space matrices or {num,den} for transfer'...
         '                       function. Then it must be discrete time.'...
          '                   * For frequency domain data Focus can also be a'...
         '                       column vector of the same size as data.Freq. This'...
         '                       gives the weights for the fit in the frequency'...
         '                       domain.'...
         ' '...
         ' MaxIter      - Maximum number of iterations for iterative search methods.'...
         '                Assignable values:'...
         '                   * A positive integer.'...
         '                     (0 gives the result of a special start-up procedure.)'...
         '                   Default: 20'...
         ' '...
         ' Tolerance    - A tolerance number for terminating the iterative search.'...
         '                When the expected improvement is less than ''Tolerance'' %'...
         '                the iterations are terminated.',...
         '                Assignable values:'...
         '                   * A positive number'...
         '                   Default: 0.01'...
         ' '...
         ' LimitError   - A value that limits the influence of large residuals'...
         '                (robustification).'...
         '                Residuals, (model errors) with a value larger than'...
         '                ''LimitError''*(estimated residual standard deviation)'...
         '                will carry a linear rather than quadratic cost.'...
         '                Assignable values:'...
         '                   * A positive number'...
         '                   * 0 means no robustification'...
         '                   Default: 1.6.'...
         ' '...
         ' MaxSize      - The size (number of elements) of the largest matrix'...
         '                 to be formed by the algorithm. If larger '... 
         '                matrices are needed, loops will be used.'...
         '                Use this value for memory/speed trade-off.'...
         '                Note: The original data matrix of u and y must be'...
         '                smaller than MaxSize',...
         '                Assignable values:'...
         '                   * Any positive integer.'...
         '                   Default: ''Auto'', means that the number is '...
         '                       determined by the m-file IDMSIZE'...
         ' '...
         ' SearchDirection - The search direction used by the iterative algorithm.'...
         '                   Assignable values:'...
         '                      * ''Gns''. The Subspace Gauss-Newton direction'...
         '                      * ''Gn''.  The Gauss-Newton direction'...
         '                      * ''Lm''.  The Levenberg-Marquardt direction'...
         '                      * ''Auto'' (default) An automatic choice of the above'...
         ' '...
         ' FixedParameters - Parameters that should remain fixed during the iterative'...
         '                   criterion minimization.'...
         '                   Assignable Values:'...
         '                      * A row vector of indices'...
         '                      * A cell array of parameter names. These will be'...
         '                        matched with model.PName. The wild card ''*'''...
         '                        can be used for termination, so that ''ps3*'' will'...
         '                        fix all parameters that start with ''ps3''.'...
		 '                        The wildcard ''?'' is used to mark an arbitrary character'...
		 '                        at the position in question. See also SETPNAME.'...
         '                       Default: []'...
         ' '...
         ' Trace           - Determines information about the minimization'...
         '                   furnished to the screen.'...
         '                   Assignable values:'...
         '                      * ''Off'' (default) No info printed'...
         '                      * ''On'' Info about criterion values given'...
         '                      * ''Full'' Info about parameters and criterion'...
         '                          values given'...
         ' '...
         ' N4Weight         - Variables that determine algorithmic details'...
         ' N4Horizons         for the N4SID method. See Help N4SID.'...
         ' '...
         ' Advanced         - A structure that contains algorithmic details',...
         '                    for the iterative search. See the manual.'...
         ' '...
         ' Note that all these properties can be accessed by GET and SET and'...
         ' subscripting using autofill for unambiguous property names, like '...
         ' ''model.se'' for the SearchDirection.'...
         ' '...
         ' See IDHELP CHANNELS for how to manipulate the channels.'...
      };
case 'estimation'
   DisplayStr = {...
         ' Help on properties associated with estimation result for' ...
         ' IDMODEL objects.'...
         ' '...
         ' The property ''EstimationInfo'' contains the following fields:'...
         ' '...
         ' Status:          Describes whether the model is estimated or not'...
         ' LossFcn:         The value of the loss function, det(sum epsi*epsi^T)'...
         ' FPE:             The Value of Akaike''s Final Prediction Error'...
         ' DataName:        Name of the data set for which the model was estimated'...
         ' DataLength:      Length of the data set used'...
         ' DataTs:          The sampling interval of the data set'...
         ' DataDomain:      Estimation data domain: ''Time'' or ''Frequency'''...
         ' DataInterSample: The intersample behavior of the input data'...
         '                  ''zoh'' means ''zero-order-hold'' (piecewise constant),'...
         '                  ''foh'' means ''first-order-hold'' (piecewise linear)'...
         '                  ''bl'' means ''bandlimited'' (no power above Nyquist freq)'...
         ' WhyStop:         For iterative search methods: The reason for termination'...
         ' UpdateNorm:      The norm of the search vector (GN-vector)'...
         ' LastImprovement: The criterion improvement in the last iteration, measured in %'...
         ' Iterations:      Number of iterations used'...
         ' N4Weight:        The actual weighting used by the N4SID method.'...
         ' N4Horizons:      The actual values of the prediction horizons used by N4SID.'...
		 ' InitialState:    The actual value of InitialState used in the estimation.'...
      }
case 'idpoly'
   
   DisplayStr ={...
         ' Properties of the IDPOLY object.'...
         ' '...
         ' IDPOLY models describe the polynomial model'...
         '  A(q)y(t) = [B(q)/F(q)] u(t-nk) + [C(q)/D(q)] e(t)'...
         ' with the special cases:'...
         '  ARX:                A(q) y(t) = B(q) u(t-nk) + e(t)'...
         '  ARMAX:              A(q) y(t) = B(q) u(t-nk) + C(q) e(t)'...
         '  OE (Output Error): y(t) = [B(q)/F(q)] u(t-nk) + e(t)'...
         '  BJ (Box-Jenkins):   y(t) = [B(q)/F(q)] u(t-nk) + [c(q)/D(q)] e(t)'...
         ' '...
         ' Properties that can be SET and GET are those of the general IDMODEL',...
         ' object (type ''idprops idmodel'') and the following ones:'...
         ' '...
         ' a:     The A-polynomial, a row vector [1 a_1 a_2 ... a_na]'...
         '        corresponding to A(q)y(t) = y(t) + a_1 y(t-1) + ... + a_na y(t-na)'...
         ' b:     The B-polynomial, a matrix with row k describing the'...
         '        B-polynomial from input no k'...
         ' c, d:  The C- and D-polynomials, similar to A'...
         ' f:     The F-polynomial, similar to B'...
         ' '...
         ' da, db, dc etc - The standard deviations of a, b, c, etc.'...
         ' '...
         ' na:    The order of the A-polynomial (number of A-coefficients'...
         ' nb:    The orders of the B-polynomials. A row vector where entry k'...
         '        gives the order of the polynomial from input number k'...
         ' nc, nd, nf: Similar for the C- D- and F-polynomials'...
         ' nk:    The input delays. A row vector where entry k gives the delay'...
         '        from input number k.'...
         ' '...
         ' InitalState: Describes how to initialize the filtering in the model'...
         '           Assignable values:'... 
         '             ''Auto'':    (default) An automatic choice of the below'...
         '             ''Zero'':     All unknown initial values are taken as zero'...
         '             ''Estimate'': The initial states are estimated from data'...
         '             ''Backcast'': Initial states are determined by backwards'...
         '                           filtering (according to Box-Jenkins and'...
         '                           T. Knudsen)'...
         ' '...
         ' The properties can be GET and SET by sub-referencing using autofill'...
         'as in A = mod.a  or  mod.ini = ''zer''.'...
      };
case 'idss'
   DisplayStr ={...
         ' Properties of the IDSS model object.'...
         ' '...
         ' IDSS describes models in State-space innovations form:',...
         ' x(t+Ts) = A x(t) + B u(t) + K e(t);    x(0)=x0'...
         '    y(t) = C x(t) + D u(t) + e(t)'...
         ' '...
         ' Properties that can be SET and GET are those of the general IDMODEL',...
         ' object (type ''idprops idmodel'') and the following ones:'...
         ' '...
         ' A, B, C, D, K, X0  - The matrices of the state description'...
         ' '...
         ' dA,DB,dC,dD,DK,dX0 - The standard deviations of A, B, etc.'...
         ' '...
         ' SSParameterization - Describes how the matrices are parameterized'...
         '        Assignable values:'...
         '        * ''Free'' (default) means that all parameters in A, B, and C'...
         '            are freely adjustable. Whether D, K, and X0 are free depends'...
         '            on the properties ''nk'', ''DisturbanceModel'',and ''InitialState''.'...
         '        * ''Canonical'': A, B, and C are parameterized as an observability'...
         '            canonical form. The form is determined by the'...
         '            ''CanonicalIndices''.'...
         '            The parameterization of D, K, and X0 depends on the properties'...
         '            ''nk'', ''DisturbanceModel'',and ''InitialState''.'...
         '        * ''Structured'': Allows arbitrarily structured matrices.'...
         '            The free and fixed parameters are determined by the structure'...
         '            matrices As, Bs, Cs, Ds, Ks, and X0s. See below.'...
         ' '...
         ' CanonicalIndices - A row vector of length ny (no of outputs) adding up'...
         '        to the model order. The indices determine the particular'...
         '        canonical parameterization in the ''Canonical'' case'...
         '        Default is ''Auto'', which gives an automatic choice'...
         ' '...
         ' As, Bs, Cs, Ds, Ks, X0s - The structure matrices for the state matrices'...
         '        A NaN in any position marks a free parameter to be estimated'...
         '        A numerical value in any position denotes a fixed parameter'...
         '        These matrices are for the user to set in the ''Structured'' case'...
         '        but are automatically generated in the other cases.'...
         '        Example: As = [1 NaN;0 NaN], A = [1 2;0 3] means that the 1,2 and'...
         '        2,2 elements of A are to be estimated, with nominal/initial'...
         '        values 2 and 3. See also IDDEMO #6.'...
         ' '...
         ' StateName - Contains the names of the state variables.'...
         '             An Nx-by-1 cell array of strings.'...
         ' '...
         ' DisturbanceModel - Determines the parameterization of the K-matrix'...
         '        Assignable values:'...
         '        * ''Estimate'': K is estimated'...
         '        * ''None'':     K is fixed to zero'...
         '        * ''Fixed'':    K is fixed to its current value'...
         ' '...
         ' InitialState - Determines the parameterization of X0'...
         '        Assignable values:'...
         '        * ''Estimate'': X0 is estimated as a parameter vector'...
         '        * ''Zero'':     X0 is fixed to zero'...
         '        * ''Fixed'':    X0 is fixed to its current value'...
         '        * ''Backcast'': X0 is estimated by a LS fit, but not stored'...
         '        * ''Auto'':     An automatic choice between the above'...
         ' '...
         ' nk       - Determines the input delays and the parameterization'...
         '        of the D-matrix.'...
         '        nk is a row vector, where entry k determines the delay from'...
         '        input number k.'...
         '        nk = [0 0 0] thus means a fully parameterized D-matrix'...
         '        while nk =[1 1 1] means that D is fixed to zero'...
         '        Larger values of nk gives an internal shift structure of'...
         '        A, B and D that describes the delay.'...
         '        nk cannot be assigned in the ''Structured'' case.'...  
         ' '...
         ' The properties can be GET and SET by sub-referencing using autofill as in'...
         ' A = mod.a  or  mod.ss = ''can''.'...
      };
   
   
case 'idarx'
   % TF-specific properties
   DisplayStr = {...
         ' Help on properties of IDARX models.'...
         ' '...
         ' The model is'...
         ' A0*y(t)+A1*y(t-T)+ ... + An*y(t-nT) ='...
         '           B0*u(t)+B1*u(t-T)+Bm*u(t-mT) + e(t)'...
         ' '...
         ' In addition to the general IDMODEL properties (type "idprops" for'...
         ' details),all IDARX models share the following IDARX-specific properties,'...
         ' listed here for a model with Ny outputs and Nu inputs:'...
         ' '....
         ' A   - The A polynomial, A Ny-by-Ny-by-(n+1) array with'...
         '       A(:,:,k+1) = Ak in the model above. Note that the normalization'...
         '       is such that A0 = eye(Ny)'...
         ' B   - The B-polynomial. A Ny-by-Nu-by-(m+1) array analogous to A'...
         '       For a time series (no input) B = [].'...
         ' '...
         ' dA, dB - The standard deviations of A and B.'...
         ' '...
         ' na  - A Ny-by-Ny matrix defining the orders of the entries' ...
         '         of the A(q) matrix polynomial. The i-j entry of na is' ...
         '         the order of the polynomial (in the delay operator)' ...
         '         that relates the j:th output the the i:th output.'...
         ' nb  - A Ny-by-Nu matrix defining the orders of the' ...
         '          entries of the B-polynomial in an analogous way.'...
         ' nk  - A Ny-by-Nu matrix, whose i-j entry defines the' ...
         '          delay from input j to output i.'...
      };
   
case 'idgrey'
   % Idgrey
   DisplayStr = {...
         ' Help on properties of IDGREY models.'...
         ' '...
         ' IDGREY models based on user-written mfiles that define linear state'...
         ' space models in innovations form:'...
         ' '...
         '      xn(t) = A x(t) + B u(t) + K e(t) ;      x(0) = X0'...
         '       y(t) = C x(t) + D u(t) + e(t)'...
         ' '...
      ' in continuous or discrete time. Here xn(t) = x(T+Ts) in discrete time'...
         ' and xn(t) = d/dt x(t) in continuous time.'...
         ' '...	
         ' In addition to the generic IDMODEL properties (type "idprops" for details),'...
         ' all IDGREY models share the following IDGREY-specific properties.'...
         ' '...
         ' MfileName - The name of the user-written mfile. The format of this'...
         '             file should be'...
         '             [A,B,C,D,K,X0] = MfileName(Par,Ts,FileArgument)'...
         '             where Par are the free parameters, Ts is the sampling'...
         '             interval, and FileArgument is an arbitrary auxiliary'...
         '             argument.'...
         ' '...
          ' CDmfile   -  describes how the mfile handles continuous/discrete time models.'...
         '         Assignable values:'...
         '        *''c''   means that the user written mfile always returns'...
         '               the continuous time system matrices, no matter the value of Ts.'...
         '               The sampling of the system will be done by the toolbox''s'...
         '               internal algorithms, and it will be done in accordance'...
         '               with the indicated data intersample behaviour. (See idprops iddata.)'...
         '        *''cd''  means that the mfile returns the continuous time state'...
         '               space model matrices when the argument Ts = 0, and the'...
         '               discrete time model, obtained by sampling with sampling'...
         '               interval Ts when Ts > 0. In this case the user''s choice'...
         '               of sampling routines will override the toolbox''s internal'...
         '               sampling algorithms.'...
         '        *''d''   means that the mfile always returns discrete time model'...
         '               matrices that may or may not depend on the value of Ts.'...
         ' '...
         ' A,B,C,D,K,X0 - The state-space matrices.'...
         ' '...
         ' dA,dB etc    - The standard deviations of A, B etc.'...
         ' '...
         ' FileArgument - is the auxiliary argument that will be passed on to'...
         '                the mfile.'...
         ' '...
        ' StateName - Contains the names of the state variables.'...
         '             An Nx-by-1 cell array of strings.'...
         ' '...
         ' InitialState - Determines the parameterization of X0'...
         '        Assignable values:'...
         '        * ''Model''     X0 is determined as the output of the mfile'...
         '        * ''Estimate'': X0 is estimated as a free parameter vector'...
         '        * ''Zero'':     X0 is fixed to zero'...
         '        * ''Fixed'':    X0 is fixed to its current value'...
         '        * ''Backcast'': X0 is estimated as the best LS fit'...
         '        * ''Auto'':     An automatic choice between the above'...
         '        Note that the X0 vector from the mfile is overridden unless'...
         '        InitialState = ''Model'' (which is the default choice).'...
         ' '...
         ' Disturbance Model - Determines the parameterization of K'...
         '        Assignable values:'...
         '        * ''Model''     K is determined as the output of the mfile'...
         '        * ''Estimate'': K is estimated as a matrix of free parameters'...
         '        * ''None'':     K is fixed to zero'...
         '        * ''Fixed'':    K is fixed to its current value'...
         '        Note that the K-matrix from the mfile is overridden unless'...
         '        DisturbanceModel = ''Model'' (which is the default choice).'...
         ' '...
         ' The properties can be GET and SET by sub-referencing using autofill as in'...
         '    model.cd = ''c'', model.di = ''est''.'...
      };   
case 'idfrd'     
   DisplayStr = {...
         ' Help on properties of IDFRD models.'...
         ' '...
         ' IDFRD models store information about Identified Frequency Response'...
         ' Data and are produced, e.g. by the estimation routines SPA, SPAFDR and ETFE.'...
         ' '...
         ' An IDFRD model with Ny outputs and Nu inputs has the following properties:'...
         ' '...
         ' Name          - An optional property to identify the model. A string.'...
         ' '...
         ' Frequency     - A column vector of length Nf containing the'...
         '                 frequency values for the responses.'... 
         ' '...
         ' ResponseData  - Ny-by-Nu-by-Nf array, such that entry (ky,ku,kf) contains'...
         '                 the complex valued response from input ku to output ky'...
         '                 at frequency value kf.'...
         ' '...
         ' SpectrumData   -Ny-by-Ny-by-Nf array, such that entry (ky1,ky2,kf)'...
         '                 contains the cross spectrum between the additive noise'...
         '                 at outputs ky1 and ky2. For a time series model (Nu = 0)'...
         '                 SpectrumData is the Spectrum matrix of the signal.'...
         ' '...
         ' CovarianceData - Ny-by-Nu-by-Nf-by-2-by-2 array such that the two last'...
         '                  dimensions contain the covariance matrix of the corre-'...
         '                  sponding ResponseData estimate. The real and imaginary'...
         '                  parts are treated as separate estimates.'...
         ' '...
         ' NoiseCovariance - Ny-by-Ny-by-Nf array such that the diagonal elements'...
         '                 at a given frequency contain the variance of the corre-'...
         '                 sponding SpectrumData element.'...
         ' '...
         ' Units           - the frequency unit. ''rad/s'' or ''Hz'''...
         ' '...
         ' Ts            - Sample time for the underlying model'...
         '                 Assignable values:'...
         '                   * Positive scalar (sampling period) for discrete models.'...
         '                   * 0 for continuous models.'...
         '                 Default value:  1.'...
         '  '...
         ' InputDelay    - A column vector of the same length as the number of'...
         '                 inputs. Defines the shifts that should be applied to'...
         '                 the input sequence to obtain the desired response.'...
         '                 Note that unlike the property ''nk'' for IDPOLY and IDSS'...
         '                 InputDelay does not show up in the model matrices and'...
         '                 transfer functions. For discrete time models (Ts>0)'... 
         '                 InputDelay is gives as number of samples, while for'...
         '                 continuous time models it is measured in TimeUnit.'...
         '                 Negative InputDelay gives non-causal models.'...
         '   '...
         ' InputName     - Input channel names.'...
         '                 Content:  Nu-by-1 cell array of strings.'...
         '                 Assignable values:'...
         '                   * String for single-input models, e.g., ''torque'''...
         '                   * Cell vector of strings for multiple-input models,'...
         '                     e.g., {''thrust'' ; ''aileron deflection''}'...
         '                   Default names:  {''u1'';''u2''; ...}.'...
         '  '...
         ' OutputName    - Output channel names.'...
         '                 Content:  Ny-by-1 cell array of strings.'...
         '                 Assignable/default values: Analogous to InputName.'...
         '  '...
         ' InputUnit     - Input channel units.'...
         '                 Content:  Nu-by-1 cell array of strings.'...
         '                 Assignable values:'...
         '                   * String for single-input models, e.g., ''m/s'''...
         '                   * Cell vector of strings for multiple-input models,'...
         '                     e.g., {''^o C'' ; ''Newton''}'...
         '                   Default names:  Empty strings.'...
         '  '...
         ' OutputUnit    - Output channel units.'...
         '                 Content:  Ny-by-1 cell array of strings.'...
         '                 Assignable/default values: Analogous to InputUnit.'...
         '  '...
         ' Notes         - User''s notes.'...
         '                 Content:  Any string or cell array of strings.'...
         '                 Example:  ''This model was created on Jan. 1st, 2000'''...
         '  '...
         ' Userdata      - Additional information or data.'...
         '                 Content:  Any MATLAB data type.'...
         ' '...
         ' Use SET and GET to set and access property values, as in'...
         '    set(model,''inputname'',''u''), get(model,''Responsedata'')'...
         ' The properties can also be GET and SET by sub-referencing using'...
         ' autofill as in loglog(mod.fr,squeeze(abs(mod.re(2,1,:)))'...
         ' See IDHELP CHANNELS for how to manipulate the channels.'...
         ' See IDFRD/CHGUNITS for changing units and IDFRD/FSELECT and IDFRD/FCAT'...
         ' for subselecting and extending responses at various frequencies.'...
         
   };
case 'iddata'
   DisplayStr = {...
         ' Help on properties of IDDATA objects.'...
         ' '...
         ' The IDDATA object stores information about data sets, suitable for'...
         ' bookkeeping and identification. '...
         ' '...
         ' An IDDATA set with Ny outputs and Nu inputs and Ne experiments has'...
         ' the following properties:'...
         ' '...
         ' Domain        - Assumes the values ''Time'' or ''Frequency'' and denotes'...
         '                 whether data data are measured in the time or frequency'...
         '                 domain.'...
         ' '...
         ' Name          - An optional property to identify the set. A string.'...
         ' '...
         ' InputData     - A N-by-Nu matrix of data such that the ky:th column'...
         '                 corresponds to the ky:th input channel.'...
         '                 For multiexperiment IDDATA, this is a 1-by-Ne cell array'...
         '                 of Ni-by-Nu matrices.'...
         '                 ''InputData'' can be referred to as ''u''.'... 
         ' InputName     - Input channel names.'...
         '                 Content:  Nu-by-1 cell array of strings.'...
         '                 Assignable values:'...
         '                   * String for single-input models, e.g., ''torque'''...
         '                   * Cell vector of strings for multiple-input models,'...
         '                     e.g., {''thrust'' ; ''aileron deflection''}'...
         '                   Default names:  {''u1'';''u2''; ...}.'...
         ' InputUnit     - Input channel units.'...
         '                 Content:  Nu-by-1 cell array of strings.'...
         '                 Assignable values:'...
         '                   * String for single-input models, e.g., ''m/s'''...
         '                   * Cell vector of strings for multiple-input models,'...
         '                     e.g., {''^o C'' ; ''Newton''}'...
         '                   Default names:  Empty strings.'...
         ' Period        - The periodicity of the inputs, in samples. For non-periodic'...
         '                 data, Period = inf. An Nu-by-1 vector. For multiple'...
         '                 experiment IDDATA a 1-by-Ne cell array.'...
         ' InterSample   - An Nu-by-1 cell array containing the intersample'...
         '                 characteristics of the inputs. Possible values:'...
         '                 ''zoh'' (zero-order-hold) The input is piecewise constant'...
         '                          between the samples.'...
         '                 ''foh'' (first-order-hold) The input is piecewise linear'...
         '                          between the samples.'...
         '                 ''bl''  (band limited) The input has no frequencies above'...
         '                          the Nyquist frequency.'...
         ' '...
         ' OutputData    - A N-by-Ny matrix of data such that the ky:th column'...
         '                 corresponds to the ky:th output channel.'...
         '                 For multiexperiment IDDATA, this is a 1-by-Ne cell array'...
         '                 of Ni-by-Ny matrices.'...
         '                 ''OutputData'' can be referred to as ''y''.'...
         ' OutputName    - Output channel names.'...
         '                 Content:  Ny-by-1 cell array of strings.'...
         '                 Assignable/default values: Analogous to InputName.'...
         ' OutputUnit    - Output channel units.'...
         '                 Content:  Ny-by-1 cell array of strings.'...
         '                 Assignable/default values: Analogous to InputUnit.'...
         '  '...
         ' Ts            - Sampling Interval. A positive scalar. For non-equal'...
         '                 sampling Ts =[ ]. For multiexperiment IDDATA, Ts is a'...
         '                 1-by-Ne cell array containing the sampling intervals for'...
         '                 the individual experiments.'...
         '                 Also defined for Frequency Domain Data, referring to'...
         '                 underlying data sampling frequency. In that case Ts=0'...
         '                 is allowed, indicating that the data values are computed'...
         '                 by the continuous time Fourier Transform.'...
         ' Tstart        - Starting time for the data. Scalar. For multiexperiment'...
         '                 IDDATA a cell array analogous to Ts. Only defined for'...
         '                 time domain data.'...
         ' SamplingInstants - A column vector of length N containing the time'...
         '                 values for the samples. For equidistant sampling this'...
         '                 is not stored, but constructed from Ts and Tstart.'...
         '                 For multiple experiment IDDATA, this is a 1-by-Ne'...
         '                 cell array containing the sampling instants of the different'...
         '                 experiments. Only defined for Time Domain data.'...
         ' TimeUnit      - The unit for the above time measures.'...
         ' '...
         ' Frequency     - A column vector of length N containing the frequency'...
         '                 values for the data samples. For multiple experiment data'...
         '                 this is a 1-by-Ne cell array. Only defined for Frequency'...
         '                 domain data.'...
         ' Units         - The unit for the Frequency property. Only defined for'...
         '                 Frequency Domain data.'...
         ' ExperimentName -A 1-by-Ne cell array containing the names of the different'...
         '                 experiments in IDDATA.'...                
         ' Notes         - User''s notes.'...
         '                 Content:  Any string or cell array of strings.'...
         '                 Example:  ''This data set was created on Jan. 1st, 2000'''...
         '  '...
         ' Userdata      - Additional information or data.'...
         '                 Content:  Any MATLAB data type.'...
         ' '...
         ' Use SET and GET to set and access property values, as in'...
         ' set(data,''inputname'',''current''), get(data,''outputdata'')'...
         ' The properties can also be GET and SET by sub-referencing using'...
         ' autofill as in  plot(data.sa,data.y)'...
         ' '...
         ' See HELP IDDATA and IDHELP CHANNELS for how to manipulate the channels.'...
      };
case 'idproc'
   % Idproc
   DisplayStr = {...
         ' Help on properties of IDPROC models.'...
         ' '...
         ' IDPROC models describe simple process models in continuous time.'...
         ' They are of the type gain + time constant + delay as in'...
         '           Kp'...
         ' G(s) = ---------  exp(-Td*s)'...
         '        1 + Tp1*s'...
         ' '...
         ' A more general case has up to three real poles and a possible zero:'...
         '                     1+Tz*s'...
         ' G(s) = Kp -------------------------- exp(-Td*s)'...
         '           (1+Tp1*s)(1+Tp2*s)(1+Tp3*s)'...
         ' '...
         ' Two of the poles may be underdamped (complex)'...
         '                     1+Tz*s'...
         ' G(s) = Kp --------------------------------- exp(-Td*s)'...
         '           (1+2*Zeta*Tw*s+(s*Tw)^2)(1+Tp3*s)'...
         ' '...
         ' In all these cases an integration may be enforced, as in'...
         '            Kp'...
         ' G(s) = ------------  exp(-Td*s)'...
         '        s(1 + Tp1*s)'...
         ' '...
         ' The details of the model are defined by the following properties'...
         ' '...
         ' Kp, Tp1, Tp2, Tp3, Tw, Zeta, Tz, Td  - These are the gains and'...
         '           time constants in the models above. They are all structues'...
         '           with the fields:'...
         '           - status: One of ''estimate'', ''zero'' or ''fixed''.'...
         '               ''estimate'' means the the parameter is (to be) estimated'...
         '               ''zero'' that the parameter is deactivated (fixed to 0)'...
         '               ''fixed'' that the parameter is fixed to its value.'...
         '           - value: the numerical value of the paramater.'...
         '           - min: the lower bound for this parameter when estimated'...
         '           - max: the upper bound for this parameter when estimated'...
         '           For multi-input models these fields are cell arrays and row'...
         '           vectors containing the information for each input.'...
         ' '...	
         ' Integration - With value ''on'' or ''off'' to denote whether or not'...
         '          an integration is enforced. A cell array for multi-input'...
         '          models.'...
         ' '...
         ' InputLevel - This is a parameter that defines the offset level of the'...
         '          input signal(s). It has the same fields and syntax as Kp, Tp1 etc.'...
         '          It may be important to estimate the input offset level for input'...
         '          channels that contaien enforced integration.'...
         ' '...
         ' Type   - An acronym that gives quick access to the type of process'...
         '          model. ''Type'' is made up of a starting ''P'' followed by'...
         '          an integer denoting the total number of poles in the model'...
         '          (not counting a possible integration). Then follows in any'...
         '          order a subset of the letters '...
         '          ''D'' denoting that a delay term exp(-Td*s) is included'...
         '          ''Z'' denoting that a numerator 1+Tz*s is included'...
         '          ''U'' denoting that the two first poles are underdamped'...
         '          ''I'' denoting that an integration is enforced'...
         '          The models examplified  above would thus be ''P1D'','...
         '          ''P3ZD'', ''P3ZDU'' and ''P1DI'', respectively.'...
         '          For multiple inputs, Type is a cell array, like {''P1D'',''P2''}'...
         ' '...
         ' InitialState - Determines how to deal with initialization of the filters'...
         '        Assignable values:'...
         '        * ''Estimate'': an initial state is estimated as a free parameter'...
         '        * ''Zero'':     all filters are initialized with zeros'...
         '        * ''Backcast'': the initialization is estimated as the best LS fit'...
         '        * ''Auto'':     An automatic choice between the above'...
         ' '...
         ' DisturbanceModel - Determines presence of a model for the additive noise'...
         '       y = G(s)u + H(s)e (e white noise)'...
         '       Assignable values:'...
         '       * ''None'': H(s) is fixed to 1.'...
         '       * ''Fixed'':H(s) is fixed to a certain model.'...
         '       * ''ARMA1'': H(s) is estimated as a first order ARMA model'...
         '       * ''ARMA2'' (or ''Estimate''): H(s) is estimated as a second order ARMA model'...
         '       For an estimated or fixed model, the property DisturbanceModel is returned'...
         '       as a cell array with the first element being the type above'...
         '       and the second entry the actual model H(s) as an IDPOLY object.'...
         '       It can also be assigned as such: set(m,''Dist'',{''Fix'',MN}), where'...
         '       MN is a continuous time IDPOLY (ARMA, 1st or 2nd order) model.'...
         '       To set the DisturbanceModel directly, use set(m,''dist'',MN). The'...
         '       model will then be estimated with MN as initial value.'...
         '       Note also that m(''n''), as always, return the Disturbance (Noise) model.'...
         ' '...
         ' In addition, an IDPROC model has all IDMODEL properties (type "idprops" for details),'...
         ' The properties can be GET and SET by sub-referencing using autofill as in'...
         '    model.ty= ''P2Z'', model.Kp.min(2) = -4 (for the second input).'...
         '    m.Tp1 = 6.3 (a numerical value) is always intepreted as m.Tp1.value = 6.3'...
         '    and model.Tz = ''estimate'' (a string) is interpreted as m.Tz.status=''estimate''.'...
         '    For assigning only some channel in the multiinput case use the full syntax:'...
         '    m.ulev.status{3} = ''estimate''. (Note: ''ulev'' is short for InputLevel.)'...
     };   
case 'focus'
    DisplayStr={...
   ' Focus        - The focus of the fit when estimating the model.'...
         '                Assignable values:'...
         '                   * ''Prediction'' (default) Focus on producing good'... 
         '                       predictors'...
         '                   * ''Simulation'' Focus on making a good fit for'...
         '                       simulation (with the current input)'...
         '                       This guarantees a stable model.'...
		 '                   * ''Stability'' to enforce the resulting model to be'...
		 '                       stable, but still having a prediction focus.'...
         '                   * A matrix of the type [WL1 WH1;WL2 WH2;...]'...
         '                       where each row defines a pass band to which'...
         '                       the fit is concentrated. Note that these'...
         '                       frequencies are given in rad/s.'...
         '                   * Any SISO filter in LTI or IDMODEL format.'...
         '                       The model is fitted to the data with a frequency '...
         '                       weighting of this SISO filter''s response.'...
         '                       The filter can also be given as {A,B,C,D} for'...
         '                       state-space matrices or {num,den} for transfer'...
         '                       function. Then it must be discrete time.'...
          '                   * For frequency domain data Focus can also be a'...
         '                       column vector of the same size as data.Freq. This'...
         '                       gives the weights for the fit in the frequency'...
         '                       domain.'...
     };
otherwise
   error(sprintf('Unknown model type ''%s''.',type))
   
end

disp(' ')
disp(char(DisplayStr))

